#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import osv
from osv import fields
from tools import config
from tools.translate import _
import time
import decimal_precision as dp
import unicodedata


def elimina_tildes(s):
    """
    Funcion para eliminar las tildes de algun texto utilizando el modulo unicodedata.
    """
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn')).replace("'","")
    



class sisb_calculation_unit_type(osv.osv):

    _name = 'sisb.calculation.unit.type'
    _description = 'Calculation Unit Type'
    _columns = {
        'name':fields.char('Unit type', size=64, required=True, readonly=False, help="Unit type"),
    }

sisb_calculation_unit_type()


class sisb_calculation_unit(osv.osv):

    _name = 'sisb.calculation.unit'
    _description = 'Calculation Unit'
    _order = 'date desc'
    _columns = {
        'name':fields.char('Reference number', size=64, required=True, readonly=False, help="Reference number under the law"),
        'date': fields.date('Date', required=True, help="Date on which goes into effect the new Unit Calculation Unit"),
        'amount': fields.float('Amount',digits_compute= dp.get_precision('Amount Bs per UT'), help="Amount of the calculation unit in Bs", required=True),
        'user_id': fields.many2one('res.users', 'Responsible', readonly=True),
        'unit_type_id': fields.many2one('sisb.calculation.unit.type', 'Unit Type', required=True),
    }
    _defaults = {
        'name': lambda *a: None,
        'user_id': lambda s, cr, u, c: u,
    }


    def get_amount_cu(self, cr, uid, name_type_cu, date=False, *args):
        """
        name_type_cu: nombre del tipo de unidad de calculo
        """
        #consultar type_cu
        type_cu_obj = self.pool.get('sisb.calculation.unit.type')
        type_cu_id = type_cu_obj.search(cr, uid, [('name','=', name_type_cu)])
        type_cu_id = type_cu_id and type_cu_id[0] or False
        #Extraer valor de la unidad segun el tipo
        rate = 0
        if type_cu_id:
            date= date or time.strftime('%Y-%m-%d')
            cr.execute("SELECT amount FROM sisb_calculation_unit WHERE unit_type_id = '%s' and date <= '%s' ORDER BY date desc LIMIT 1" % (type_cu_id, date))
            if cr.rowcount:
                rate=cr.fetchall()[0][0]
        return rate

    def compute(self, cr, uid, from_amount, name_type_cu, date=False, context={}):
        result = 0.0
        cu = self.get_amount_cu(cr, uid, name_type_cu, date=False)
        if cu:
            result = from_amount / cu

        return result

    def compute_cu_to_money(self, cr, uid, amount_cu, name_type_cu, date=False, context={}):
        money = 0.0
        cu = self.get_amount_cu(cr, uid, name_type_cu, date)
        if cu:
            money = amount_cu * cu
        return money

sisb_calculation_unit()

class caja_chica(osv.osv):
    _inherit = "caja.chica"
    
    #Se sobrescribe la funcion del constraint del modelo de caja.chica
    def _check_monto_caja_chica(self, cr, uid, ids, context=None):
        """ Verificar que el monto de la caja chica, no exceda la cantidad de
        unidades permitidas configurado para caja chica.
        @return: True (bien) or False (mal)
        """
        print '_check_monto_caja_chica'
        if context is None:
            context = {}
            
        lista_ut = []
        lista_uc = []
        resultado = True
        for ca_ch in self.browse(cr, uid, ids):
            
            if ca_ch.calculation_unit == 'ut':
                lista_ut.append(ca_ch.id) 
            else:
                lista_uc.append(ca_ch.id)
        
        if lista_ut:
            resultado = super(caja_chica, self)._check_monto_caja_chica(cr, uid, lista_ut, context=context)
        
        if resultado and lista_uc:
            for ca_ch in self.browse(cr, uid, lista_uc, context=context):
                if not ca_ch.config:
                    raise osv.except_osv(('Error !'), ('No existe configuracion para caja chica creada'))
                
                
                ####OBTENER CONFIGURACION DE LA UNIDAD DE CALCULO#####
                calc_unit_type = ca_ch.unit_type_id and ca_ch.unit_type_id.name or False
                cu_obj = self.pool.get('sisb.calculation.unit')
                fecha_actual = time.strftime('%Y-%m-%d')
                ut_ser = cu_obj.get_amount_cu(cr, uid, calc_unit_type, fecha_actual)
                
                if not ut_ser:
                    raise osv.except_osv(('Error !'), ('No existe unidad de calculo creada para la caja chica'))
                ###########################################################
                
                maximo_ut = float(ca_ch.config.max_reposicion)
                ut_amount = float(ut_ser)
                monto_caja = float(ca_ch.amount)
                monto_permitido = maximo_ut * ut_amount
                print 'monto_permitido',monto_permitido
                if (monto_permitido - monto_caja) < 0:
                    raise osv.except_osv(('Error !'), ('El saldo de la caja chica es superior a las %s %s permitida'%(maximo_ut, calc_unit_type)))
                    return False

            return True

        return resultado
    
    _columns = {
        'calculation_unit': fields.selection([('other','Other'),('ut','U.T')], 'Calculation Unit', required=True, readonly=True, states={'draft': [('readonly', False)]}, select=True),
        'unit_type_id': fields.many2one('sisb.calculation.unit.type', 'Unit Type', readonly=True, states={'draft': [('readonly', False)]}, select=True),
    }
    
    _defaults = {
        'calculation_unit':'other',
    }
    
    _constraints = [(_check_monto_caja_chica, 'El saldo de la caja chica es superior al monto permitido para una caja chica', ['amount'])]
    
    def onchange_calculation_unit(self, cr, uid, ids, calculation_unit, context=None):
        if calculation_unit == 'ut':
            return {'value': {'unit_type_id': False}}
        else:
            return {'value': {}}
            
caja_chica()


class caja_reposicion_linea(osv.osv):
    _inherit = "caja.reposicion.linea"
    
    def _check_linea_reposicion(self, cr, uid, ids, context=None):
        """ Verificar que las lineas de la reposicion de caja chica,
        no exceda la cantidad de unidades permitidas configurado para caja chica.
        @return: True (bien) or False (mal)
        """
        print '_check_linea_reposicion'
        if context is None:
            context = {}
        
        lista_repo_line_ut = []
        lista_repo_line_uc = []
        resultado = True
        for repo_line in self.browse(cr, uid, ids):
            
            if repo_line.caja_reposicion_id.caja_chica_id.calculation_unit == 'ut':
                lista_repo_line_ut.append(repo_line.id) 
            else:
                lista_repo_line_uc.append(repo_line.id)
        
        if lista_repo_line_ut:
            resultado = super(caja_reposicion_linea, self)._check_linea_reposicion(cr, uid, lista_repo_line_ut, context=context)
        
        if resultado and lista_repo_line_uc:
            for lin_rep in self.browse(cr, uid, lista_repo_line_uc, context=context):
                if lin_rep.amount <= 0:
                    raise osv.except_osv(('Error !'), ('Debe colocar un monto mayor a 0'))
                
                
                ####OBTENER CONFIGURACION DE LA UNIDAD DE CALCULO#####
                calc_unit_type = lin_rep.caja_reposicion_id.caja_chica_id.unit_type_id and lin_rep.caja_reposicion_id.caja_chica_id.unit_type_id.name or False
                cu_obj = self.pool.get('sisb.calculation.unit')
                fecha_actual = time.strftime('%Y-%m-%d')
                ut_ser = cu_obj.get_amount_cu(cr, uid, calc_unit_type, fecha_actual)
                
                if not ut_ser:
                    raise osv.except_osv(('Error !'), ('No existe unidad de calculo creada para la caja chica'))
                ###########################################################
                
                max_line_total = 0
                config_brw = lin_rep.caja_reposicion_id.caja_chica_id and lin_rep.caja_reposicion_id.caja_chica_id.config or False
                if not config_brw:
                    config_brw = config_obj.browse(cr, uid, [config_ser[0]])[0]
                if not config_brw:
                    raise osv.except_osv(('Error !'), ('Se debe crear una configuracion para el manejo de caja chica.'))
                    
                ut_amount = float(ut_ser)
                max_line_ut = config_brw.max_linea
                max_line_total = max_line_ut * ut_amount
                print 'max_line_total',max_line_total
                if lin_rep.amount - max_line_total > 0.0001:
                    raise osv.except_osv(('Error !'), ('La linea "%s" excede las %s de %s. No puede ser aprobada la reposicion.'%(lin_rep.name,max_line_ut,calc_unit_type)))
                    return False
                    
            return True
        
        return resultado
        
        
    _constraints = [(_check_linea_reposicion, 'El monto de la linea excede las unidades de calculo permitidas. No puede ser aprobada la reposicion.', ['amount'])]
    
    
caja_reposicion_linea()

class caja_reposicion(osv.osv):
    _inherit = "caja.reposicion"
    
    def validar_reposicion(self, cr, uid, ids, context=None):
        """
        Aqui se aprueba y se generan los asientos contables al momento de aprobar
        la reposicion.
        """
        print 'validar_reposicion'
        if context is None:
            context = {}
        
        lista_repo_ut = []
        lista_repo_uc = []
        resultado = True
        for repo in self.browse(cr, uid, ids):
            
            if repo.caja_chica_id.calculation_unit == 'ut':
                lista_repo_ut.append(repo.id) 
            else:
                lista_repo_uc.append(repo.id)
        
        if lista_repo_ut:
            resultado = super(caja_reposicion, self).validar_reposicion(cr, uid, lista_repo_ut, context=context)
        
        if resultado and lista_repo_uc:
            move_obj = self.pool.get('account.move')
            move_line_obj = self.pool.get('account.move.line')
            repo_line_obj = self.pool.get('caja.reposicion.linea')
            config_obj = self.pool.get('caja.chica.configuracion')
            obj_company = self.pool.get('res.company')
            
            repo_brw = self.browse(cr, uid, lista_repo_uc, context=context)
            
            """
            Inicio de validacion de las lineas de la caja chica para que no excedan
            el limite definido para las cajas chicas.
            """
            
            for repo in repo_brw:
                
                ####OBTENER CONFIGURACION DE LA UNIDAD DE CALCULO#####
                #~ ut_ser
                calc_unit_type = repo.caja_chica_id.unit_type_id and repo.caja_chica_id.unit_type_id.name or False
                cu_obj = self.pool.get('sisb.calculation.unit')
                fecha_actual = time.strftime('%Y-%m-%d')
                ut_ser = cu_obj.get_amount_cu(cr, uid, calc_unit_type, fecha_actual)
                
                if not ut_ser:
                    raise osv.except_osv(('Error !'), ('No existe unidad de calculo creada para la caja chica'))
                #####################################
                
                max_line_total = 0
                config_brw = repo.caja_chica_id.config
                if not config_brw:
                    config_brw = config_obj.browse(cr, uid, [config_ser[0]])[0]
                if not config_brw:
                    raise osv.except_osv(('Error !'), ('Se debe crear una configuracion para el manejo de caja chica.'))
                
                ut_amount = float(ut_ser)
                max_line_ut = config_brw.max_linea
                max_line_total = max_line_ut * ut_amount
                print 'max_line_total',max_line_total
                for line in repo.reposicion_linea_ids:
                    if line.amount - max_line_total > 0.0001:
                        raise osv.except_osv(('Error !'), ('La linea "%s" excede las %s de %s. No puede ser aprobada la reposicion.'%(lin_rep.name,max_line_ut,calc_unit_type)))
                    if line.account_id.type == 'view':
                        raise osv.except_osv(('Error !'), ('Imposible aprobar la reposicion. En la linea %s, se eligio una cuenta de tipo vista. (%s %s)' % (line.name, line.account_id.code, line.account_id.name)))
                        
            #Fin de la validacion
            
            for repo in repo_brw:
                caja_brw = repo.caja_chica_id
                move_dict = {}
                ref = elimina_tildes(repo.name + ' / ' + repo.ref)
                move_dict = {
                    'ref': ref,
                    'journal_id': caja_brw.journal_move_id and caja_brw.journal_move_id.id or None,
                    'to_check': True,
                }
                move_id = move_obj.create(cr, uid, move_dict)
                move_brw = move_obj.browse(cr, uid, [move_id])[0]
                periodo_id = move_brw.period_id.id
                context.update({'period_id': periodo_id})
                
                move_line_ids = []
                company_id = obj_company._company_default_get(cr, uid, 'account.move', context=context)
                for line in repo.reposicion_linea_ids:
                    move_line_dict = {}
                    name = elimina_tildes(line.name + ' / ' + line.concepto)
                    move_line_dict = {
                        'name': name[:64],
                        'ref': ref[:64],
                        #~ 'quantity': ,
                        #~ 'product_uom_id': ,
                        'product_id': line.product_id and line.product_id.id or None,
                        'debit': line.amount,
                        'credit': 0.00,
                        'account_id': line.account_id and line.account_id.id or None,
                        'move_id': move_id,
                        'journal_id': caja_brw.journal_move_id and caja_brw.journal_move_id.id or None,
                        'analytic_account_id': line.analytic_id and line.analytic_id.id or 'NULL',
                        'state': 'valid',
                        'period_id': periodo_id,
                    }
                    
                    #Creacion de apunte contable para debito por medio de query
                    cr.execute("insert into account_move_line (account_id,name,ref,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,product_id,analytic_account_id) values (%s,'%s','%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now(),%s,%s)"%(move_line_dict['account_id'],move_line_dict['name'],move_line_dict['ref'],move_line_dict['journal_id'],int(company_id),move_line_dict['credit'],move_line_dict['state'],move_line_dict['period_id'],move_line_dict['debit'],str(fecha_actual),move_line_dict['move_id'],uid,move_line_dict['product_id'],move_line_dict['analytic_account_id']))
                    cr.commit()

                    move_line_dict.update({'debit': 0.00, 'credit': line.amount, 'account_id': caja_brw.account_id and caja_brw.account_id.id or None, 'partner_id': repo.company_id.partner_id.id})

                    #Creacion de apunte contable para credito por medio de query
                    #se incorpora el partner en el apunte
                    cr.execute("insert into account_move_line (account_id,name,ref,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,product_id,analytic_account_id,partner_id) values (%s,'%s','%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now(),%s,%s,%s)"%(move_line_dict['account_id'],move_line_dict['name'],move_line_dict['ref'],move_line_dict['journal_id'],int(company_id),move_line_dict['credit'],move_line_dict['state'],move_line_dict['period_id'],move_line_dict['debit'],str(fecha_actual),move_line_dict['move_id'],uid,move_line_dict['product_id'],move_line_dict['analytic_account_id'],move_line_dict['partner_id']))
                    cr.commit()
                    
                    #relacionar la linea de la reposicion con el apunte contable asociado a la caja chica
                    apunte_id = move_line_obj.search(cr, uid, [('move_id', '=',move_id),('partner_id','!=',False)], context=context)
                    if apunte_id:
                        dict_linea = {
                            'move_line_id':apunte_id[0],
                        }
                        repo_line_obj.write(cr, uid, [line.id], dict_linea, context=context)
                    else:
                        raise osv.except_osv(('Error !'), ('Error in create the move.'))
                        
                #Asentar el asiento contable de la caja chica.
                move_obj.button_validate(cr, uid, [move_id], context=context)
                
                #Unir lineas de asiento a la reposicion
                move_line_ids = [mo.id for mo in move_brw.line_id]
                self.write(cr, uid, [repo.id], {'move_ids': [(6, 0, move_line_ids)]}, context=context)
                
                
            self.write(cr, uid, lista_repo_uc, {'state': 'done', 'aprobado_por': uid, 
                'fecha_aprobado': fecha_actual, 'period_id': periodo_id}, context=context)
            for repo in self.browse(cr, uid, lista_repo_uc):
                context.update({'estado': repo.state})
                self.log_reposicion(cr, uid, repo.id, context=context)
            
            return True
        
        
        return resultado
        
    
caja_reposicion()
